window.Gleam.initSearch([{"doc":"ask","title":"ask","content":"# ask\n\n[![Package Version](https://img.shields.io/hexpm/v/ask)](https://hex.pm/packages/ask)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/ask/)\n![CI](https://github.com/furrycatherder/gleam-ask/actions/workflows/tests.yml/badge.svg?branch=main)\n\n`ask` is a Gleam library tailored for simplifying the management of\nequivalences, orderings, and predicates. Equivalences allow for the comparison\nof two values to determine if they&#39;re equivalent, while orderings provide a\nsystematic way to compare values in terms of their relative magnitudes or\npositions. Predicates, on the other hand, assess if a given value meets\nspecific conditions, offering a flexible mechanism for expressing logical\nconstraints or requirements.\n\n## Overview\n\n`ask` provides developers with a set of functions to create, combine, and\ntransform equivalences, orderings, and predicates. Whether you&#39;re performing\nbasic value comparisons or intricate logical operations, `ask` furnishes the\nnecessary tools to handle a variety of scenarios effectively.\n\n## Installation\n\n```sh\ngleam add ask\n```\n\n## Usage\n\n### Equivalences\n\n```gleam\nimport ask/eq\n\n// Defining custom equivalences\nlet eq1 = fn(x, y) { x % 3 == y % 3 }\nlet eq2 = fn(x, y) { x % 5 == y % 5 }\n\n// Combining equivalences using logical operations\nlet combined_eq = eq.and(eq1, eq2)\n\n// Using equivalence to compare values\nlet result = combined_eq(15, 30) // Returns True\n```\n\nEquivalences are expected to follow these friendly rules:\n\n- **Reflexivity**: Every value is equivalent to itself. It&#39;s like saying, &quot;Hey,\n  you&#39;re always equal to yourself!&quot;\n- **Symmetry**: If one value is equivalent to another, then it&#39;s a two-way\n  street! If X is like Y, then Y is like X. It&#39;s all about fairness!\n- **Transitivity**: Imagine a chain of equivalence! If X is equivalent to Y,\n  and Y is equivalent to Z, then it&#39;s like saying X is also buddies with Z.\n  Friendship circles all around!\n\nThese rules help keep our equivalences reliable and predictable, making sure\nthey play nice with each other.\n\n### Orderings\n\n```gleam\nimport ask/ord\nimport gleam/int\nimport gleam/list\nimport gleam/string\n\ntype User {\n  User(name: String, age: Int)\n}\n\nlet compare_name = ord.map_input(string.compare, fn(user: User) { user.name })\nlet compare_age = ord.map_input(int.compare, fn(user: User) { user.age })\n\n// Sort by name, then by age\nlet compare_user = ord.combine(compare_name, compare_age)\nlet result =\n  list.sort(\n    [\n      User(name: &quot;alice&quot;, age: 32),\n      User(name: &quot;bob&quot;, age: 45),\n      User(name: &quot;alice&quot;, age: 24),\n    ],\n    by: compare_user,\n  )\n```\n\n### Predicates\n\n```gleam\nimport ask/predicate\n\n// Defining custom predicates\nlet is_positive = fn(x) { x &gt; 0 }\nlet is_even = fn(x) { x % 2 == 0 }\n\n// Combining predicates using logical operations\nlet combined_pred = predicate.and(is_positive, is_even)\n\n// Using predicate to evaluate values\nlet result = combined_pred(6) // Returns True\nlet result = combined_pred(-6) // Returns False\n```\n\n## Conclusion\n\n`ask` equips Gleam developers with a robust toolkit for managing equivalences,\norderings, and predicates efficiently. Whether you&#39;re implementing algorithms,\ndata validation systems, or decision-making processes, `ask` facilitates\nstreamlined code development, allowing you to focus on problem-solving.\n\nFurther documentation can be found at &lt;https://hexdocs.pm/ask&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the tests\ngleam shell # Run an Erlang shell\n```\n","url":"index.html"},{"doc":"ask/eq","title":"Equivalence","content":"pub type Equivalence(a) =\n  fn(a, a) -&gt; Bool\n\n\nSynonyms:\nask/eq.Equivalence\nask/eq Equivalence","url":"ask/eq.html#Equivalence"},{"doc":"ask/eq","title":"and","content":"pub fn and(\n  first: fn(a, a) -&gt; Bool,\n  second: fn(a, a) -&gt; Bool,\n) -&gt; fn(a, a) -&gt; Bool\n Combine two equivalences into a new equivalence that returns `True` if both\n equivalences return `True`.\n\nSynonyms:\nask/eq.and\nask/eq and","url":"ask/eq.html#and"},{"doc":"ask/eq","title":"contains","content":"pub fn contains(eq: fn(a, a) -&gt; Bool) -&gt; fn(List(a), a) -&gt; Bool\n Test if a value is a member of a list according to the given equivalence.\n\nSynonyms:\nask/eq.contains\nask/eq contains","url":"ask/eq.html#contains"},{"doc":"ask/eq","title":"default","content":"pub fn default() -&gt; fn(a, a) -&gt; Bool\n Create a new equivalence that uses the default equality comparison.\n\nSynonyms:\nask/eq.default\nask/eq default","url":"ask/eq.html#default"},{"doc":"ask/eq","title":"list","content":"pub fn list(eq: fn(a, a) -&gt; Bool) -&gt; fn(List(a), List(a)) -&gt; Bool\n Create a new equivalence for a list of values based on the given equivalence.\n\nSynonyms:\nask/eq.list\nask/eq list","url":"ask/eq.html#list"},{"doc":"ask/eq","title":"map_input","content":"pub fn map_input(\n  over eq: fn(a, a) -&gt; Bool,\n  with fun: fn(b) -&gt; a,\n) -&gt; fn(b, b) -&gt; Bool\n Map the input of an equivalence to create a new equivalence.\n\n ## Examples\n\n This can be useful for constructing a custom equivalence of a record, given\n that we have an equivalence for one or more of its fields.\n\n \n import ask/equivalence as eq\n\n type User {\n   User(name: String, age: Int)\n }\n\n pub fn main() {\n   let user1 = User(&quot;alice&quot;, 30)\n   let user2 = User(&quot;alice&quot;, 24)\n   let is_username_equal =\n     eq.default()\n     |&gt; eq.map_input(fn(user: User) { user.name })\n\n   is_username_equal(user1, user2) // -&gt; True\n }\n \n\nSynonyms:\nask/eq.map_input\nask/eq map_input","url":"ask/eq.html#map_input"},{"doc":"ask/eq","title":"negate","content":"pub fn negate(eq: fn(a, a) -&gt; Bool) -&gt; fn(a, a) -&gt; Bool\n Negate an equivalence to create a new equivalence that returns `True` if the\n original equivalence returns `False`.\n\nSynonyms:\nask/eq.negate\nask/eq negate","url":"ask/eq.html#negate"},{"doc":"ask/eq","title":"or","content":"pub fn or(\n  first: fn(a, a) -&gt; Bool,\n  second: fn(a, a) -&gt; Bool,\n) -&gt; fn(a, a) -&gt; Bool\n Combine two equivalences into a new equivalence that returns `True` if either\n equivalence returns `True`.\n\nSynonyms:\nask/eq.or\nask/eq or","url":"ask/eq.html#or"},{"doc":"ask/eq","title":"pair","content":"pub fn pair(\n  first: fn(a, a) -&gt; Bool,\n  second: fn(b, b) -&gt; Bool,\n) -&gt; fn(#(a, b), #(a, b)) -&gt; Bool\n Create a new equivalence for a pair of values based on the given equivalences.\n\nSynonyms:\nask/eq.pair\nask/eq pair","url":"ask/eq.html#pair"},{"doc":"ask/eq","title":"trivial","content":"pub fn trivial() -&gt; fn(a, a) -&gt; Bool\n Create a new equivalence that always returns `True`.\n\n This is sometimes called the trivial or universal equivalence.\n\nSynonyms:\nask/eq.trivial\nask/eq trivial","url":"ask/eq.html#trivial"},{"doc":"ask/eq","title":"unique","content":"pub fn unique(eq: fn(a, a) -&gt; Bool) -&gt; fn(List(a)) -&gt; List(a)\n Remove duplicates from a list, keeping the first occurrence of each element\n according to the given equivalence.\n\nSynonyms:\nask/eq.unique\nask/eq unique","url":"ask/eq.html#unique"},{"doc":"ask/eq","title":"ask/eq","content":"","url":"ask/eq.html"},{"doc":"ask/ord","title":"Ord","content":"pub type Ord(a) =\n  fn(a, a) -&gt; order.Order\n\n\nSynonyms:\nask/ord.Ord\nask/ord Ord","url":"ask/ord.html#Ord"},{"doc":"ask/ord","title":"between","content":"pub fn between(\n  ord: fn(a, a) -&gt; Order,\n  lower: a,\n  upper: a,\n) -&gt; fn(a) -&gt; Bool\n Test whether a value is between two others, according to a given ordering.\n\n The provided interval is half-open, `[lower, upper)`, i.e. greater than or\n equal to lower and less than upper.\nSynonyms:\nask/ord.between\nask/ord between","url":"ask/ord.html#between"},{"doc":"ask/ord","title":"clamp","content":"pub fn clamp(\n  ord: fn(a, a) -&gt; Order,\n  lower: a,\n  upper: a,\n) -&gt; fn(a) -&gt; a\n Clamp a value to a given range, according to a given ordering.\n\nSynonyms:\nask/ord.clamp\nask/ord clamp","url":"ask/ord.html#clamp"},{"doc":"ask/ord","title":"combine","content":"pub fn combine(\n  first: fn(a, a) -&gt; Order,\n  second: fn(a, a) -&gt; Order,\n) -&gt; fn(a, a) -&gt; Order\n Combine two orderings into a new ordering, which will order by the first,\n then by the second in case of a tie.\n\nSynonyms:\nask/ord.combine\nask/ord combine","url":"ask/ord.html#combine"},{"doc":"ask/ord","title":"get_equivalence","content":"pub fn get_equivalence(\n  ord: fn(a, a) -&gt; Order,\n) -&gt; fn(a, a) -&gt; Bool\n Every law-abiding ordering is also an equivalence.\n\nSynonyms:\nask/ord.get_equivalence\nask/ord get_equivalence","url":"ask/ord.html#get_equivalence"},{"doc":"ask/ord","title":"map_input","content":"pub fn map_input(\n  over ord: fn(a, a) -&gt; Order,\n  with fun: fn(b) -&gt; a,\n) -&gt; fn(b, b) -&gt; Order\n Map the input of an ordering to create a new ordering.\n\nSynonyms:\nask/ord.map_input\nask/ord map_input","url":"ask/ord.html#map_input"},{"doc":"ask/ord","title":"pair","content":"pub fn pair(\n  first: fn(a, a) -&gt; Order,\n  second: fn(b, b) -&gt; Order,\n) -&gt; fn(#(a, b), #(a, b)) -&gt; Order\n Create a new ordering for a pair of values based on the given orderings.\n\nSynonyms:\nask/ord.pair\nask/ord pair","url":"ask/ord.html#pair"},{"doc":"ask/ord","title":"trivial","content":"pub fn trivial() -&gt; fn(a, a) -&gt; Order\n Create a new ordering that always returns `order.Eq`.\n\nSynonyms:\nask/ord.trivial\nask/ord trivial","url":"ask/ord.html#trivial"},{"doc":"ask/ord","title":"ask/ord","content":"","url":"ask/ord.html"},{"doc":"ask/predicate","title":"Predicate","content":"pub type Predicate(a) =\n  fn(a) -&gt; Bool\n\n\nSynonyms:\nask/predicate.Predicate\nask/predicate Predicate","url":"ask/predicate.html#Predicate"},{"doc":"ask/predicate","title":"all","content":"pub fn all(p: fn(a) -&gt; Bool) -&gt; fn(List(a)) -&gt; Bool\n Create a new predicate that returns `True` if the given predicate returns\n `True` for every element in a list.\n\nSynonyms:\nask/predicate.all\nask/predicate all","url":"ask/predicate.html#all"},{"doc":"ask/predicate","title":"always","content":"pub fn always() -&gt; fn(a) -&gt; Bool\n Create a new predicate that always returns `True`.\n\nSynonyms:\nask/predicate.always\nask/predicate always","url":"ask/predicate.html#always"},{"doc":"ask/predicate","title":"and","content":"pub fn and(\n  first: fn(a) -&gt; Bool,\n  second: fn(a) -&gt; Bool,\n) -&gt; fn(a) -&gt; Bool\n Combine two predicates together into a new predicate that returns `True` if\n both predicates return `True`.\n\nSynonyms:\nask/predicate.and\nask/predicate and","url":"ask/predicate.html#and"},{"doc":"ask/predicate","title":"any","content":"pub fn any(p: fn(a) -&gt; Bool) -&gt; fn(List(a)) -&gt; Bool\n Create a new predicate that returns `True` if the given predicate returns\n `True` for any element in a list.\n\nSynonyms:\nask/predicate.any\nask/predicate any","url":"ask/predicate.html#any"},{"doc":"ask/predicate","title":"every","content":"pub fn every(ps: List(fn(a) -&gt; Bool)) -&gt; fn(a) -&gt; Bool\n Combine a list of predicates together into a new predicate that returns `True`\n if all predicates return `True`.\n\nSynonyms:\nask/predicate.every\nask/predicate every","url":"ask/predicate.html#every"},{"doc":"ask/predicate","title":"map_input","content":"pub fn map_input(\n  over p: fn(a) -&gt; Bool,\n  with fun: fn(b) -&gt; a,\n) -&gt; fn(b) -&gt; Bool\n Map the input of a predicate to create a new predicate.\n\nSynonyms:\nask/predicate.map_input\nask/predicate map_input","url":"ask/predicate.html#map_input"},{"doc":"ask/predicate","title":"negate","content":"pub fn negate(p: fn(a) -&gt; Bool) -&gt; fn(a) -&gt; Bool\n Negate a predicate.\n\nSynonyms:\nask/predicate.negate\nask/predicate negate","url":"ask/predicate.html#negate"},{"doc":"ask/predicate","title":"never","content":"pub fn never() -&gt; fn(a) -&gt; Bool\n Create a new predicate that always returns `False`.\n\nSynonyms:\nask/predicate.never\nask/predicate never","url":"ask/predicate.html#never"},{"doc":"ask/predicate","title":"or","content":"pub fn or(\n  first: fn(a) -&gt; Bool,\n  second: fn(a) -&gt; Bool,\n) -&gt; fn(a) -&gt; Bool\n Combine two predicates together into a new predicate that returns `True` if\n either predicate returns `True`.\n\nSynonyms:\nask/predicate.or\nask/predicate or","url":"ask/predicate.html#or"},{"doc":"ask/predicate","title":"some","content":"pub fn some(ps: List(fn(a) -&gt; Bool)) -&gt; fn(a) -&gt; Bool\n Combine a list of predicates together into a new predicate that returns `True`\n if any predicate returns `True`.\n\nSynonyms:\nask/predicate.some\nask/predicate some","url":"ask/predicate.html#some"},{"doc":"ask/predicate","title":"ask/predicate","content":"","url":"ask/predicate.html"}]);